@@ -1,6 +1,6 @@
 package TPath::Compiler;
 {
-  $TPath::Compiler::VERSION = '0.009';
+  $TPath::Compiler::VERSION = '0.010';
 }
 
 # ABSTRACT: takes ASTs and returns compiled L<TPath::Expression> objects
@@ -82,11 +82,23 @@ sub cs {
 	my $q = $step->{cs}{quantifier};
 	if ( exists $step->{cs}{step} ) {
 		my $s = full( $step->{cs}, $forester );
+		return Quantified->new(
+			s          => $s,
+			quantifier => 'e',
+			top        => $q->{enum}{end},
+			bottom     => $q->{enum}{start}
+		) if ref $q;
 		return Quantified->new( s => $s, quantifier => $q );
 	}
 	my $e = SE->new( e => treepath( $step->{cs}{grouped_step}, $forester ) );
 	return $e unless $q;
-	return Quantified->new( e => $e, quantifier => $q );
+	return Quantified->new(
+		s          => $e,
+		quantifier => 'e',
+		top        => $q->{enum}{end},
+		bottom     => $q->{enum}{start}
+	) if ref $q;
+	return Quantified->new( s => $e, quantifier => $q );
 }
 
 sub full {
@@ -383,7 +395,7 @@ TPath::Compiler - takes ASTs and returns compiled L<TPath::Expression> objects
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 DESCRIPTION
 
