@@ -1,6 +1,6 @@
 package TPath::Selector::Quantified;
 {
-  $TPath::Selector::Quantified::VERSION = '0.009';
+  $TPath::Selector::Quantified::VERSION = '0.010';
 }
 
 # ABSTRACT: handles expressions like C<a?> and C<//foo*>
@@ -20,21 +20,44 @@ has s => ( is => 'ro', isa => 'TPath::Selector', required => 1 );
 
 has quantifier => ( is => 'ro', isa => 'Quantifier', required => 1 );
 
+
+has top => ( is => 'ro', isa => 'Int', default => 0 );
+
+
+has bottom => ( is => 'ro', isa => 'Int', default => 0 );
+
 sub select {
 	my ( $self, $n, $i, $first ) = @_;
 	my @c = $self->s->select( $n, $i, $first );
 	for ( $self->quantifier ) {
 		when ('?') { return @c, $n }
-		when ('*') { return @{ _iterate( $self->s, $first, $i, \@c ) }, $n }
-		when ('+') { return @{ _iterate( $self->s, $first, $i, \@c ) } }
+		when ('*') { return @{ _iterate( $self->s, $i, \@c ) }, $n }
+		when ('+') { return @{ _iterate( $self->s, $i, \@c ) } }
+		when ('e') {
+			my ( $s, $top, $bottom ) =
+			  ( $self->s, $self->top, $self->bottom );
+			my $c = _enum_iterate( $s, $i, \@c, $top, $bottom, 1 );
+			return @$c, $self->bottom < 2 ? $n : ();
+		}
 	}
 }
 
+sub _enum_iterate {
+	my ( $s, $i, $c, $top, $bottom, $count ) = @_;
+	my @next = map { $s->select( $_, $i ) } @$c;
+	my @return = $count++ >= $bottom ? @$c : ();
+	unshift @return, @next
+	  if $count >= $bottom && ( !$top || $count <= $top );
+	unshift @return, @{ _iterate( $s, $i, \@next, $top, $bottom, $count ) }
+	  if !$top || $count < $top;
+	return \@return;
+}
+
 sub _iterate {
-	my ( $s, $first, $i, $c ) = @_;
+	my ( $s, $i, $c ) = @_;
 	return [] unless @$c;
-	my @next = map { $s->select( $_, $i, $first ) } @$c;
-	return [ @{ _iterate( $s, $first, $i, \@next ) }, @$c ];
+	my @next = map { $s->select( $_, $i ) } @$c;
+	return [ @{ _iterate( $s, $i, \@next ) }, @$c ];
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -51,7 +74,7 @@ TPath::Selector::Quantified - handles expressions like C<a?> and C<//foo*>
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 DESCRIPTION
 
@@ -67,6 +90,15 @@ The selector to which the quantifier is applied.
 
 The quantifier.
 
+=head2 top
+
+The largest number of iterations permitted. If 0, there is no limit. Used only by
+the C<{x,y}> quantifier.
+
+=head2 bottom
+
+The smallest number of iterations permitted. Used only by the C<{x,y}> quantifier.
+
 =head1 ROLES
 
 L<TPath::Selector>
